#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class ProjectPriority(Enum):
    HIGH = "HIGH"
    LOW = "LOW"
    MEDIUM = "MEDIUM"
    NONE = "NONE"
    URGENT = "URGENT"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "ProjectPriority":
        return cls.MISSING_ENUM
