#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.filter_operator import FilterOperator
from ..enum.rule_type_filter_type import RuleTypeFilterType


@dataclass(frozen=True)
class RuleTypeFilterInput(DataClassJsonMixin):
    filterType: RuleTypeFilterType = _field(metadata=enum_field_metadata(RuleTypeFilterType))
    idSet: List[str]
    operator: FilterOperator = _field(metadata=enum_field_metadata(FilterOperator))
    stringSet: List[str]
    maxDepth: Optional[int] = None
    stringValue: Optional[str] = None
