#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.survey_question_type import SurveyQuestionType


@dataclass(frozen=True)
class SurveyTemplateQuestionInput(DataClassJsonMixin):
    index: int
    questionDescription: str
    questionTitle: str
    questionType: SurveyQuestionType = _field(metadata=enum_field_metadata(SurveyQuestionType))
    id: Optional[str] = None
