#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class FlowInstanceStatus(Enum):
    CANCELED = "CANCELED"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"
    IN_PROGRESS = "IN_PROGRESS"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "FlowInstanceStatus":
        return cls.MISSING_ENUM
