#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class ServiceFilterType(Enum):
    EQUIPMENT_IN_SERVICE = "EQUIPMENT_IN_SERVICE"
    LOCATION_INST = "LOCATION_INST"
    LOCATION_INST_EXTERNAL_ID = "LOCATION_INST_EXTERNAL_ID"
    PROPERTY = "PROPERTY"
    SERVICE_DISCOVERY_METHOD = "SERVICE_DISCOVERY_METHOD"
    SERVICE_INST_CUSTOMER_NAME = "SERVICE_INST_CUSTOMER_NAME"
    SERVICE_INST_EXTERNAL_ID = "SERVICE_INST_EXTERNAL_ID"
    SERVICE_INST_NAME = "SERVICE_INST_NAME"
    SERVICE_STATUS = "SERVICE_STATUS"
    SERVICE_TYPE = "SERVICE_TYPE"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "ServiceFilterType":
        return cls.MISSING_ENUM
