#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from ..input.parent_property_type_value_input import ParentPropertyTypeValueInput


@dataclass(frozen=True)
class AddPropertyTypeValueInput(DataClassJsonMixin):
    name: str
    id: Optional[str] = None
    isDeleted: Optional[bool] = None
    parentPropertyType: Optional[List[ParentPropertyTypeValueInput]] = None
    parentPropertyTypeValue: Optional[List[str]] = None
    propertyType: Optional[str] = None
