#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from gql_client.runtime.enum_utils import enum_field_metadata
from ..enum.survey_question_type import SurveyQuestionType

from ..input.file_input import FileInput
from ..input.survey_cell_scan_data import SurveyCellScanData
from ..input.survey_wi_fi_scan_data import SurveyWiFiScanData


@dataclass(frozen=True)
class SurveyQuestionResponse(DataClassJsonMixin):
    cellData: List[SurveyCellScanData]
    formIndex: int
    imagesData: List[FileInput]
    questionIndex: int
    questionText: str
    wifiData: List[SurveyWiFiScanData]
    altitude: Optional[Number] = None
    boolData: Optional[bool] = None
    dateData: Optional[int] = None
    emailData: Optional[str] = None
    floatData: Optional[Number] = None
    formDescription: Optional[str] = None
    formName: Optional[str] = None
    intData: Optional[int] = None
    latitude: Optional[Number] = None
    locationAccuracy: Optional[Number] = None
    longitude: Optional[Number] = None
    phoneData: Optional[str] = None
    photoData: Optional[FileInput] = None
    questionFormat: Optional[SurveyQuestionType] = None
    textData: Optional[str] = None
