#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class RecommendationsFilterType(Enum):
    ALARMTYPE = "ALARMTYPE"
    COMMAND = "COMMAND"
    EXTERNALID = "EXTERNALID"
    LONGDESCRIPTION = "LONGDESCRIPTION"
    PRIORITY = "PRIORITY"
    RECOMMENDATIONSCATEGORY = "RECOMMENDATIONSCATEGORY"
    RECOMMENDATIONSSOURCE = "RECOMMENDATIONSSOURCE"
    RESOURCE = "RESOURCE"
    RUNBOOK = "RUNBOOK"
    SHORTDESCRIPTION = "SHORTDESCRIPTION"
    STATUS = "STATUS"
    USED = "USED"
    USERAPPROVE = "USERAPPROVE"
    USERCREATE = "USERCREATE"
    VENDORRECOMMENDATIONS = "VENDORRECOMMENDATIONS"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "RecommendationsFilterType":
        return cls.MISSING_ENUM
