#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config


@dataclass(frozen=True)
class AddCostInput(DataClassJsonMixin):
    externalId: str
    item: str
    position: int
    price: Number
    quantity: int
    total: Number
    type_price: str
    type_unit: str
    unit: Number
    uplitem: str
    id: Optional[str] = None
    isDeleted: Optional[bool] = None
    workorder: Optional[str] = None
