#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class YesNoResponse(Enum):
    NO = "NO"
    YES = "YES"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "YesNoResponse":
        return cls.MISSING_ENUM
