#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from dataclasses import dataclass, field as _field
from functools import partial
from ...config import custom_scalars, datetime
from numbers import Number
from typing import Any, AsyncGenerator, Dict, List, Generator, Optional

from dataclasses_json import DataClassJsonMixin, config

from ..input.action_block_input import ActionBlockInput
from ..input.connector_input import ConnectorInput
from ..input.decision_block_input import DecisionBlockInput
from ..input.end_block_input import EndBlockInput
from ..input.goto_block_input import GotoBlockInput
from ..input.start_block_input import StartBlockInput
from ..input.subflow_block_input import SubflowBlockInput
from ..input.trigger_block_input import TriggerBlockInput
from ..input.true_false_block_input import TrueFalseBlockInput
from ..input.variable_definition_input import VariableDefinitionInput


@dataclass(frozen=True)
class ImportFlowDraftInput(DataClassJsonMixin):
    actionBlocks: List[ActionBlockInput]
    connectors: List[ConnectorInput]
    decisionBlocks: List[DecisionBlockInput]
    endBlocks: List[EndBlockInput]
    endParamDefinitions: List[VariableDefinitionInput]
    gotoBlocks: List[GotoBlockInput]
    id: str
    name: str
    subflowBlocks: List[SubflowBlockInput]
    triggerBlocks: List[TriggerBlockInput]
    trueFalseBlocks: List[TrueFalseBlockInput]
    description: Optional[str] = None
    startBlock: Optional[StartBlockInput] = None
