#!/usr/bin/env python3
# @generated AUTOGENERATED file. Do not Change!

from enum import Enum


class PropertyKind(Enum):
    bool = "bool"
    date = "date"
    datetime_local = "datetime_local"
    email = "email"
    enum = "enum"
    float = "float"
    gps_location = "gps_location"
    int = "int"
    node = "node"
    range = "range"
    string = "string"
    MISSING_ENUM = ""

    @classmethod
    def _missing_(cls, value: object) -> "PropertyKind":
        return cls.MISSING_ENUM
